/*
   include.c -- PIKT piktc include routines

   $Header: /cvs/PIKT/pikt/src/piktc/include.c,v 1.11 2001/10/10 04:08:00 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#include "../lib/common.h"
#include "piktc.h"

/* open an input stream, returning a pointer to that stream, else a null
   pointer if the open fails
 */
FILE *
instrmopen (int inctyp, char *infile, char *inproc, int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "instrmopen";
	FILE *instrm;
		
	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "in %s(), pid is %d, ppid is %d, incnum is %d, called from %s, line %d", fnnam, getpid(), getppid(), incnum, srcfile, lno);

	instrm = (FILE *) NULL;
	switch (inctyp) {
		case INCFILE:
			if (! (instrm = fopen(infile, "r")))
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), fopen() error for: %s", fnnam, infile);
			break;
		case INCPROC:
			if (! (instrm = popen(inproc, "r")))
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), popen() error for: %s", fnnam, inproc);
			else
				sleep(fork_delay);
			break;
		default:
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), unknown inctype %d", fnnam, inctyp);
			break;
	}
	return instrm;
}

/* close an input stream
 */
int
instrmclose (int inctyp, FILE *instrm, int incnum, char *srcfile, int lno)
{
	static char fnnam[] = "instrmclose";
		
	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "in %s(), pid is %d, ppid is %d, incnum is %d, called from %s, line %d", fnnam, getpid(), getppid(), incnum, srcfile, lno);

	switch (inctyp) {
		case INCFILE:
			if (fclose(instrm))
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, TRUE, "in %s(), fclose() error", fnnam);
			break;
		case INCPROC:
			if (pclose(instrm))
				/*
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, TRUE, "in %s(), pclose() error", fnnam);
				 */
				;
			break;
		default:
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), unknown inctype %d", fnnam, inctyp);
			return ERROR;
			break;
	}
	return 0;
}

/* when the lexer processes #include and/or #verbatim directives, determine
   which is which, returning TRUE if #verbatim, FALSE if #include
 */
int
isverbatim (char *yytxt)
{
	register int i;

	i = 0;
	while (! isalpha((int)*(yytxt+i)))
		i++;
	if (*(yytxt+i) == 'v')
		return TRUE;
	else
		return FALSE;
}

/* check for existence of #include|#verbatim file, and if nonexistent,
   create an empty one; if we don't create an empty placeholder file,
   the lexer/parser gets confused and zeroes out subsequent #include|
   #verbatim <*.cfg> [<proc>] *.cfg files
 */
void
check_incfile (int inctyp, char *file)
{
	static char fnnam[] = "check_incfile";
	struct stat fbuf;

	if (inctyp == INCFILE) {
		if (file) {
			if (stat(file, &fbuf) != 0) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), stat() error for %s", fnnam, file);
				if (! create_incfile(file)) {
					bailout = TRUE;
					return;
				}
			}
		} else {
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), include file unspecified", fnnam);
			bailout = TRUE;
			return;
		}
	} else if (inctyp == INCPROC) {
		if (file) {
			if (stat(file, &fbuf) != 0) {
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, TRUE, "in %s(), stat() error for %s", fnnam, file);
				if (! create_incfile(file)) {
					bailout = TRUE;
					return;
				}
			}
		} /* else there is no file to worry about, because
		     inctyp == INCPROC, and this is an include of
		     the form '#include [<proc>]'
		   */
	}
	return;
}

/* create an empty #include file */
int
create_incfile (char *file)
{
	static char fnnam[] = "create_incfile";
	FILE *inctmp;

	/* use LOG_ERR in the following logreturn() in order that the message
	   gets sent to stdout
	 */
	logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), creating empty file %s", fnnam, file);
	if ((inctmp = fopen(file, "w")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fopen() error for %s", fnnam, file);
		bailout = TRUE;
		return FALSE;
	}
	fprintf(inctmp, "%s", "");
	fflush(inctmp);
	if (fclose(inctmp)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fclose() error for %s", fnnam, file);
		bailout = TRUE;
		return FALSE;
	}
	if (chmod(file, (mode_t)(S_IRUSR|S_IWUSR|S_IRGRP))) {	/* 640 */
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), chmod() error for %s", fnnam, file);
		bailout = TRUE;
		return FALSE;
	}
	return TRUE;
}
