/*
   log.c -- PIKT piktc & piktc_svc log routines

   $Header: /cvs/PIKT/pikt/src/piktc/log.c,v 1.27 2001/10/10 04:08:00 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#include "../lib/common.h"
#include "piktc.h"

#include <rpc/rpc.h>
#include "rpc.h"

const char *leveltypes[] = { "EMERG", "ALERT", "CRIT", "ERR", "WARNING", "NOTICE", "INFO", "DEBUG" };

int syslog_facility = LOG_LOCAL0;

static void logit(int, int, const char *, int, int, const char *, va_list ap);

/* nonfatal error; log the message, then return to the caller
 */
void
logreturn (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, ...)
{
	va_list ap;

	#ifdef PIKTC_SVC
	if (/* ! logdir || */ ! piktc_svclog) {
	#else
	if (/* ! logdir || */ ! piktclog) {
	#endif
		if ((logfiletype != SYSLOG) || ! isautosyslog) {
			return;
		}
	}
	va_start(ap, fmt);
	logit(logfiletype, logtype, srcfile, lno, syscall, fmt, ap);
	va_end(ap);
	return;
}

/* fatal error; log the message, then exit the program
 */
void
logexit (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, ...)
{
	va_list ap;

	#ifdef PIKTC_SVC
	if (/* ! logdir || */ ! piktc_svclog) {
	#else
	if (/* ! logdir || */ ! piktclog) {
	#endif
		if ((logfiletype != SYSLOG) || ! isautosyslog) {
			#ifdef PIKTC_SVC
			svc_unregister(PIKTCPROG, PIKTCVERS);
			rmlocks(FALSE, FALSE);
			#else
			rmlocks(TRUE, FALSE);
			#endif
			exit(2);
		}
	}
	va_start(ap, fmt);
	logit(logfiletype, logtype, srcfile, lno, syscall, fmt, ap);
	va_end(ap);
	#ifdef PIKTC_SVC
	svc_unregister(PIKTCPROG, PIKTCVERS);
	rmlocks(FALSE, FALSE);
	#else
	rmlocks(TRUE, FALSE);
	#endif
	exit(2);
}

/* format the message, adding a date/time stamp, a label, source file &
   source line number, if a serious error, and system error message,
   as appropriate; print the message, either to a file or to stderr
 */
static void
logit (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, va_list ap)
{
	static char fnnam[] = "logit";
	int errno_save;
	char buf[MAXBUF+1];	/* arbitrarily large */
	struct tm *tm;
	time_t now;
	FILE *logfile;

	errno_save = errno;
	now = time((time_t*)0);
	tm = localtime(&now);
	strcpy(buf, asctime(tm));
	switch (logtype) {
		case LOG_ERR:
			sprintf(buf+20, "ERROR: (%s, line %d) ", srcfile, lno);
			break;
		default:
			sprintf(buf+20, "%s: ", leveltypes[(logtype != NIL) ? logtype : LOG_DEBUG]);
			break;
	}
	vsprintf(buf+strlen(buf), fmt, ap);
	#if defined(SHOW_SYSCALL)
	if (syscall)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	#endif
	strcat(buf, "\n");
	/* if we ever have to deal with DQTPROXY/SQTPROXY:
	requote(buf);
	 */
	switch (logfiletype) {
		case PIKTLOG:
			#ifdef PIKTC_SVC
			if ((logfile = fopen(piktc_svclog, "a")) == NULL) {
				fflush(stdout);
				fprintf(stderr, "ERROR: in %s(), fopen error for %s\n", fnnam, piktc_svclog);
				fflush(stderr);
				rmlocks(FALSE, FALSE);
				exit(1);
			}
			fputs(buf+4, logfile);
			if (fclose(logfile)) {
				fflush(stdout);
				fprintf(stderr, "ERROR: in %s(), fclose() error for %s\n", fnnam, piktc_svclog);
				fflush(stderr);
				rmlocks(FALSE, FALSE);
				exit(1);
			}
			chmod(piktc_svclog, (mode_t)(S_IRUSR|S_IWUSR));	/* 600 */
			#else
			if (isdebug) {
				fflush(stdout);
				fputs(buf+4, stderr);
				fflush(stderr);
			} else {
				if ((logfile = fopen(piktclog, "a")) == NULL) {
					if (isverbose) {
						fflush(stdout);
						fprintf(stderr, "ERROR: in %s(), fopen() error for %s\n", fnnam, piktclog);
						fflush(stderr);
					}
					rmlocks(TRUE, FALSE);
					exit(1);
				}
				fputs(buf+4, logfile);
				/* for client, which is interactive, send error
				   message also to stderr
				 */
				if (isverbose && (logtype == LOG_ERR)) {
					fflush(stdout);
					fputs(buf+4, stderr);
					fflush(stderr);
				}
				if (fclose(logfile)) {
					if (isverbose) {
						fflush(stdout);
						fprintf(stderr, "ERROR: in %s(), fclose() error for %s\n", fnnam, piktclog);
						fflush(stderr);
					}
					rmlocks(TRUE, FALSE);
					exit(1);
				}
				chmod(piktclog, (mode_t)(S_IRUSR|S_IWUSR));	/* 600 */
			}
			#endif
			break;
		#ifdef PIKTD
		case LOCLOG:
		case SYSLOG:
			/* piktd only */
		#endif
		default:
			break;
	}
	if (isautosyslog) {
		#ifdef PIKTC_SVC
		openlog("piktc_svc", LOG_CONS|LOG_PID, syslog_facility);
		#else
		openlog("piktc", LOG_CONS|LOG_PID, syslog_facility);
		#endif
		syslog((logtype != NIL) ? logtype : LOG_DEBUG, "%s", buf+20);
		closelog();
	}
	return;
}
